package edu.ntnu.stud.chaosgame.controller.game;

import javafx.scene.shape.Rectangle;

public class QuadtreeNode {
  private QuadtreeNode parent;
  private QuadtreeNode[] children;

  /**
   * Boolean representing whether the node is a leaf node or
   * a child node.
   */
  private boolean isLeaf;

  /**
   * Integer describing which recursion level this node belongs to.
   */
  private int recursionLevel;
  private Rectangle bounds;

  private double minX;
  private double minY;
  private double maxX;
  private double maxY;


  public QuadtreeNode(Rectangle bounds, int recursionLevel) {
    this.bounds = bounds;
    this.minX = bounds.getX();
    this.minY = bounds.getY() - bounds.getHeight();
    this.maxX = bounds.getX() + bounds.getWidth();
    this.maxY = bounds.getY();

    this.recursionLevel = recursionLevel;
    this.isLeaf = true;
    this.children = new QuadtreeNode[4];
  }

  /**
   * Split this parent node into four child nodes.
   */
  public void split() {
    if (!isLeaf) {
      return;
    }

    double halfWidth = bounds.getWidth() / 2;
    double halfHeight = bounds.getHeight() / 2;

    children[0] = new QuadtreeNode(new Rectangle(bounds.getX(), bounds.getY(), halfWidth, halfHeight), recursionLevel
        + 1);
    children[1] = new QuadtreeNode(new Rectangle(bounds.getX() + halfWidth, bounds.getY(), halfWidth, halfHeight), recursionLevel
        + 1);
    children[2] = new QuadtreeNode(new Rectangle(bounds.getX(), bounds.getY() + halfHeight, halfWidth, halfHeight), recursionLevel
        + 1);
    children[3] = new QuadtreeNode(new Rectangle(bounds.getX() + halfWidth, bounds.getY() + halfHeight, halfWidth, halfHeight), recursionLevel
        + 1);

    isLeaf = false;
  }

  /**
   * Merge this node with its three counterparts to form a single child node.
   */
  public void merge() {
    if (isLeaf) {
      return;
    }

    for (int i = 0; i < 4; i++) {
      children[i] = null;
    }

    isLeaf = true;
  }

  /**
   * Check if this node contains or is adjacent to a specified point in the plane.
   *
   * @param centerX the centre of the point in the x-axis.
   * @param centerY the centre of the point in the y-axis.
   * @return a boolean representing the result o the check.
   */
  public boolean containsOrAdjacent(double centerX, double centerY) {
    // Check if the point is within the node
    if (centerX >= this.minX && centerX <= this.maxX && centerY >= this.minY && centerY <= this.maxY) {
      return true;
    }

    // Check if the point is adjacent to the node
    if (centerX >= this.minX - 1 && centerX <= this.maxX + 1 && centerY >= this.minY - 1 && centerY <= this.maxY + 1) {
      return true;
    }

    return false;
  }

  public Rectangle getBounds() {
    return this.bounds;
  }

  public QuadtreeNode[] getChildren() {
    return this.children;
  }

  public Boolean isLeaf() {
    return this.isLeaf;
  }

  // getters and setters omitted for brevity
}
